/*--------------------------
 *
 * This module is for the Sopho Sweep
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "compute.h"
#define INITGUID
#include "csavi2c.h"
#undef INITGUID

#include "return_codes.h"
#include "mod_sweep.h"

/* internal only global variables */
CISavi2 *pSAVI;

int SWEEP_start( void ){
	int result = _ERR_NO_STATUS;

	CISweepClassFactory2 *pFactory;
	HRESULT hr;
	const char *ClientName = "SAVIDemo";

	 hr = DllGetClassObject((REFIID)&SOPHOS_CLSID_SAVI2,
                         (REFIID)&SOPHOS_IID_CLASSFACTORY2,
                         (void **)&pFactory);

	if (SOPHOS_FAILED(hr)) 
		fprintf(stderr,"Error initialising Class object\n");

	if (hr == SOPHOS_S_OK ) {
		hr=pFactory->pVtbl->CreateInstance(pFactory,\
			NULL,\
			&SOPHOS_IID_SAVI2,\
			(void **)&pSAVI );
			
		pFactory->pVtbl->Release(pFactory);
	
		if (hr == SOPHOS_S_OK){
			hr=pSAVI->pVtbl->InitialiseWithMoniker(pSAVI, ClientName);
		
			if (SOPHOS_FAILED(hr)) {
			fprintf(stderr,"Error: Initialise [%ld].",(long)hr);
			pSAVI->pVtbl->Release(pSAVI);
			pSAVI= NULL;
			} /* if fail */
			
  /* Turn on Full SWEEP */

  hr = pSAVI->pVtbl->SetConfigValue(pSAVI,
                                    SOPHOS_DO_FULL_SWEEP,
                                    SOPHOS_TYPE_U32,
                                    _T("1"));
  if (SOPHOS_FAILED(hr))
  {
    _tprintf(_T("ERROR: Unable to set config value for Full SWEEP [0x%08lx].\n"), (unsigned long)hr);
    /* if the error is SOPHOS_SAVI2_ERROR_NOT_SUPPORTED then don't bail */
    if(hr != SOPHOS_SAVI2_ERROR_NOT_SUPPORTED)
    _tprintf(_T("       Not supported for this platform.\n"));
  }

  /* Turn on ZIP decompression */

  hr = pSAVI->pVtbl->SetConfigValue(pSAVI,
                                    SOPHOS_ZIP_DECOMPRESSION,
                                    SOPHOS_TYPE_U32,
                                    _T("1"));
  if (SOPHOS_FAILED(hr))
  {
    _tprintf(_T("ERROR: Unable to set config value for ZIP decompression [0x%08lx].\n"), (unsigned long)hr);
    /* if the error is SOPHOS_SAVI2_ERROR_NOT_SUPPORTED then don't bail */
    if(hr != SOPHOS_SAVI2_ERROR_NOT_SUPPORTED)
    _tprintf(_T("       Not supported for this platform.\n"));
  }


			
		} /* if sophos ok */
	} /* if sophos ok */
			
	return result;
	}


	
int SWEEP_scan_file( char *fname ){
	int result = _ERR_NO_STATUS;
	HRESULT hr;
	
	hr = pSAVI->pVtbl->SweepFile( pSAVI , fname, (REFIID)&SOPHOS_IID_ENUM_SWEEPRESULTS, NULL );

#ifdef DEBUG
	fprintf(stderr,">>Result = %d\n",hr);	
#endif
	switch (hr) {
		case SOPHOS_S_OK: result = _ERR_OK; break;
		case SOPHOS_SAVI2_ERROR_VIRUSPRESENT: result = _ERR_VIRUS_FOUND; break;
		default : result = _ERR_NO_STATUS;
		}
		
	return result;
	}

int SWEEP_close( void ) {
	
	if (pSAVI){
		pSAVI->pVtbl->Terminate(pSAVI);
		pSAVI->pVtbl->Release(pSAVI);
		}

	return 0;
	}

