#include <stdio.h>
#include <stdlib.h>
#include "ebola_api.h"
#include "return_codes.h"

#define _USAGE "\n\
EbolaClient usage:\n\
ebolaclient -s <servername/ip> <-u user> <-p password> -f <file list> [-d output_file] [-o ebola port]\n\
\n\
	-s The name or IP of your local server which is running the Ebola daemon (usually localhost etc)\n\
	-u Your user name to authenticate to Ebola with\n\
	-p Your password for your username\n\
	-f The name of the file containing the LIST of files you want to have scanned by Ebola\n\
	-d [optional] If specified, this is the file which will contain a verbose description of the scan\n\
	-o [optional] If Ebola is not running on port 1665, then specify which port here.\n\
	-h This text.\n\
\n"

/* systemwide parameter specs */	
char *servername = NULL;
char *user = NULL;
char *password = NULL;
char *filelist = NULL;
char *outputfile = NULL;
int port = 1665;


/*
 * The main call
 *
 */
int main( int argc, char **argv ){
	
	char line[1024];
	FILE *f, *fo=NULL;
	int i=0, result = 0, gresult = 0;

	/* if we dont have the minimal number of parameters, then 
	 * show the user what we want 
	 */
	if (argc < 9) {
		fprintf(stdout,"%s",_USAGE);
		exit(1);
		}
		
		
	/* For ever parameter in our argv's, check to see if
	 * its a 'flag' (as noted with a '-' prefix) and if
	 * so, then pick up the next parameter as the value
	 * to be read in
	 */
	for (i = 1; i < argc; i++) {
		
		if (argv[i][0] == '-') { /* we have a switch */
		switch (argv[i][1]) {
			case 's': servername = argv[++i]; break;
			case 'u': user = argv[++i]; break;
			case 'p': password = argv[++i]; break;
			case 'f': filelist = argv[++i]; break;
			case 'd': outputfile = argv[++i]; break;
			case 'o': port = atoi(argv[++i]); break;
			case 'h': fprintf(stdout,"%s\n",_USAGE); break;
			default: fprintf(stderr,"ERROR, flag '%s %s' unknown.\n",argv[i], argv[++i]); exit(1); break;
			} /* switch */
			} /* if - flag */
		} /* for */		
	
	
	/* Try and open up the file list to scan */	
	f = fopen(filelist,"r");
	
	/* if the file didn't open, then we better let the enduser know */
	if (!f) {
		fprintf(stderr,"Error: could not open file '%s' for reading\n",filelist);
		exit(1);
		}  /* if !f */
	
	/* if outputfile has been defined (ie, not NULL as by default 
	 * then try open the file as 'fo'
	 */
	if (outputfile) {
		fo = fopen(outputfile,"w");
		
		/* if outputfile didn't open, then once again, lets let the
		 * end user know */
		if (!fo) {	
			fprintf(stderr,"Error: could not open file '%s' for writing\n",filelist);
			exit(1);
			} /* if !fo */
		} /* if outputfile */
				
	/* if we've gotten this far, then startup the EBOLA client
	 */
	EBOLA_start(servername, port, user, password);
	
	/* while we've still got more files to 'scan' from our
	 * input file list */
	while(fgets(line,1023,f)){
		
		/* scan and get the result */
		result = EBOLA_scan_file(line);
		
		/* if a virus is found, then add on to our global-result */
		if (result == _ERR_VIRUS_FOUND) gresult += result;
		
		/* if we have a defined output file, then dump a short 'log' 
		 * type entry into the outputfile */
		if (fo) fprintf(fo, "%s:%d\n",line, result);
		} /* while */

	/* close up our files */
	if (f) fclose(f);
	if (fo) fclose(fo);
	
	/* stop the Ebola client-socket */
	EBOLA_stop();

	/* if there was a virus detected, then return this as a result */	
	if (gresult > 0) gresult = _ERR_VIRUS_FOUND;

	return gresult;
	}
		
